#! /usr/bin/gawk -f
# ######################################################################### #
#                                                                           #
# File:     ASCII2Text.awk                                                  #
# Authors:  Brigitte Bigi                                                   #
# Date:     april, 2007                                                     #
# Licence:  GPL                                                             #
# Brief:    Get ASCII input text and convert to "local" text format         #
#                                                                           #
# ######################################################################### #
 
# ######################################################################### #
#                                                                           #
# Function Name: FCT_Usage                                                  #
#                                                                           #
# Description:                                                              #
#     Print the usage and exit the program                                  #
#                                                                           #
# ######################################################################### #
function FCT_Usage ()
{
        printf "Usage: ASCII2Text.awk [options]\n\n"
        printf "Where options include:\n"
        printf "  -o filename  write the output to this file (default: stdout)\n"
	printf "  -s           DONT use a very simplist algorithm to split sentences\n"
        printf "  -h           print this help\n"
        printf "\n"
        printf " input text:  stdin\n"
        printf " output text: stdout\n"
        exit
}


# ######################################################################### #
#                                                                           #
# Function Name: FCT_CommandLine                                            #
#                                                                           #
# Description:                                                              #
#     Get arguments from the command line                                   #
#                                                                           #
# ######################################################################### #
function FCT_CommandLine(argc, argv)
{
        entry = 0
	utf8encoding = 0
        for (i=1; i<argc; i++)
        {
                if (argv[i] == "-h")
                        FCT_Usage()
                else
		if (argv[i] == "-s")
			splittosent = 0
                else
                if (argv[i] ~ /^\-/)
                {
                        printf "invalid option -- %s\n", argv[i] > "/dev/stderr"
                        FCT_Usage()
                }
        }
}


BEGIN{
        Output      = "/dev/stdout"     # output file name
	splittisent = 1			# default
        FCT_CommandLine(ARGC, ARGV)	# extract parameters
        delete ARGV
        ARGC=1

	# Ecriture de l'entete xml
	print "\n<corpus>\n" 	> Output
	print "<doc>"		> Output
	print "<descr>"		> Output
	print "</descr>"	> Output
	print "<text>\n<><>"	> Output
}

{
        # Supprime les redondances d'espaces et espaces en debut de chaine
        # idem pour les tabulations...
        gsub("[  ]+", " ")
        gsub(/^ /, "")
        gsub(/^	/, "")

	if (length($0) > 0)
	{
		if (splittosent == 1)
		{
			# Propose une segmentation en phrases "simpliste"
			gsub (/\. /, ". <>\n")
			gsub (/\? /, " ? <>\n")
			gsub (/\! /, " ! <>\n")
		}
		printf "%s ", $0> Output
	}
	else
	{
		# Une ligne blanche...
		print " <>\n"	> Output
	}
}

END {
	print "\n</text>"	> Output
	print "</doc>"		> Output
	print "</corpus>"	> Output
}

