#! /usr/bin/gawk -f
# ######################################################################### #
#                                                                           #
# File:     Num2LetterCN.awk                                                #
# Authors:  Brigitte Bigi, Tan Ming, Sun Wenwen                             #
# Date:     Jan, 2008                                                       #
# Licence:  GPL                                                             #
# Brief:    Convert numbers of a corpus to the corresponding string(chinese)#
# Help:     Num2LetterCN.awk -h                                             #
#                                                                           #
# ######################################################################### #



# ######################################################################### #
#                                                                           #
# Function: unite                                                           #
# 	Converts numbers from 0 to 9 and returns the corresponding string   #
#                                                                           #
# ######################################################################### #
function unite( number )
{
	_r = ""
	if (number == 9)  _r = " 九 "
	if (number == 8)  _r = " 八 "
	if (number == 7)  _r = " 七 "
	if (number == 6)  _r = " 六 "
	if (number == 5)  _r = " 五 "
	if (number == 4)  _r = " 四 "
	if (number == 3)  _r = " 三 "
	if (number == 2)  _r = " 二 "
	if (number == 1)  _r = " 一 "
	if (number == 0)  
	{
		_str = " 零 "
		for(i=1; i<length(number); i++)
			_str = " " _str " 零 "
		return _str
	}
	
	return _r
}


# ######################################################################### #
#                                                                           #
# Function: dizaine                                                         #
# 	Converts numbers from 1 to 99 and returns the corresponding string  #
#                                                                           #
# ######################################################################### #
function dizaine( number )
{

	if (number < 10) _str = unite(number)

	if (number >= 10 && number < 100)
	{
		if ( (number%10) != 0 )
			_str = unite(int (number/10)) " 十 " unite(number%10)
        
		if ( (number%10) == 0 )
			_str = unite(int (number/10)) "十"
	}	

	return _str
}


# ######################################################################### #
#                                                                           #
# Function: centaine                                                        #
# 	Converts numbers from 1 to 999 and returns the corresponding string #
#                                                                           #
# ######################################################################### #
function centaine( number )
{

	if (number < 100) _str = dizaine(number)

	if (number >= 100 && number < 1000)
	{
		if ( (number%100) != 0 )
                {
                     if ( (number%100) > 0 && (number%100) < 10 )
                       _str = dizaine(int (number/100)) " 百 " "零 " dizaine(number%100)
                     else
			_str = dizaine(int (number/100)) " 百 " dizaine(number%100)
                }
        
		if ( (number%100) == 0 )
			_str = dizaine(int (number/100)) "百"
	}	

	return _str	
}	


# ######################################################################### #
#                                                                           #
# Function: millier                                                         #
# Converts numbers from 1000 to 9999 and returns the corresponding string   #
#                                                                           #
# ######################################################################### #
function millier( number )
{
	# Milliers
	# --------

	if (number < 1000) _str = centaine(number)

	if (number >= 1000 && number < 10000)
	{
		if ( (number%1000) != 0 )
                {
                     if ( (number%1000) > 0 && (number%1000) < 100 )
                       _str = centaine(int (number/1000)) " 千 "" 零 " centaine(number%1000)
                     else
			_str = centaine(int (number/1000)) " 千 " centaine(number%1000)
                }
        
		if ( (number%1000) == 0 )
			_str = centaine(int (number/1000)) "千"
	}	

	return _str
}


# ############################################################################# #
#                                                                               #
# Function: million                                                             #
# Converts numbers from 10000 to 99999999 and returns the corresponding string  #
#                                                                               #
# ############################################################################# #
    
function million( number )
{
	# de dizaine milliers à dizaine millions
	# --------------------

	if (number < 10000)
		_str = millier(number)
	
	if (number >= 10000 && number < 100000000)
	{
		if ( (number%10000) != 0 )
                {
                     if ( (number%10000) > 0 && (number%10000) < 1000 )
                       _str = millier(int (number/10000)) " 万 " " 零 " millier(number%10000)
                     else
			_str = millier(int (number/10000)) " 万 " millier(number%10000)
                }
        
		if ( (number%10000) == 0 )
			_str = millier(int (number/10000)) "万"
	}
	
	return _str
}


# ######################################################################### #
#                                                                           #
# Function: FCT_Num2text                                                    #
#	Converts all numbers and returns the corresponding string           #
#                                                                           #
#  number is the number to convert                                          #
#                                                                           #
# ######################################################################### #

function FCT_Num2text( number )
{
	if (number < 100000000)
		_str = million(number)


	# Milliards
	# --------

	if (number >= 100000000)
	{
		if ( (number%100000000) != 0 )
                {
                     if ( (number%100000000) > 0 && (number%100000000) < 10000000 )
                       _str = million(int (number/100000000)) " 亿 " " 零 " million(number%100000000)
                     else
			_str = million(int (number/100000000)) " 亿 "million(number%100000000)
                }
        
		if ( (number%100000000) == 0 )
			_str = million(int (number/100000000)) " 亿 "
	}
	
	return _str
}


# ######################################################################### #
#                                                                           #
# Function Name: FCT_Usage                                                  #
#                                                                           #
# Description:                                                              #
#     Print the usage and exit the program                                  #
#                                                                           #
# ######################################################################### #
function FCT_Usage ()
{
        printf "Usage: Num2LetterCN.awk [options]\n\n"
        printf "Where options include:\n"
	printf "  -o filename  write the output to this file (default: stdout)\n"
	printf "  -noxml       input is one word a line (instead of XML)\n"
        printf "  -h	print  this help\n"
	printf "\n"
	printf " input number: stdin\n"
	printf " output text:  stdout\n"
        exit
}


# ######################################################################### #
#                                                                           #
# Function Name: FCT_CommandLine                                            #
#                                                                           #
# Description:                                                              #
#     Get arguments from the command line                                   #
#                                                                           #
# ######################################################################### #
function FCT_CommandLine(argc, argv)
{
        entry = 0
        for (i=1; i<argc; i++)
        {
                if (argv[i] == "-h")
                        FCT_Usage()
                else
                if (argv[i] == "-noxml")
                {
                        xml = 0
                        i++
                }
                else
                if (argv[i] == "-o")
                {
                        Output = argv[i+1]
                        i++
                }
                else
		if (argv[i] ~ /^\-/)
		{
			printf "invalid option -- %s\n", argv[i] > "/dev/stderr"
			FCT_Usage()
		}
        }
}

# ######################################################################### #
#                                                                           #
# Function Name: FCT_Convert                                                #
#                                                                           #
# Description:                                                              #
#     Get a number and return a string                                      #
#                                                                           #
# ######################################################################### #

function FCT_Convert ( w )
{
	_w = w
	_strnum = ""

	# Numbers starting by one or more "0"
	if (match(_w,/^0/))
	{
		while (match(_w,/^0/))
		{
			_strnum = _strnum " 零 "
			sub(/^0/, "", _w)
		}
	}
	gsub(/\./, "",_w)
	if (length(_w) > 0)
		_strnum = _strnum FCT_Num2text(int(_w))
	return _strnum
}



# ############################# BEGIN MAIN ################################ #

BEGIN {
	Output      = "/dev/stdout"	# output file name
	xml         = 1			# xml input ??? (yes is default)
        FCT_CommandLine(ARGC, ARGV)	# extract parameters
        delete ARGV
        ARGC=1
}

/<str>/ {
        # Get the element
        s = index($0, ">")				# start: 1st >
        e = index(substr($0, s+1, length($0)), "<")	# end: 1st < after start
        w = substr($0, s+1, e-1)			# word: $0 from start to end
        gsub(/ /, "", w)				# remove spaces

	# Verify if the input string is a number
	nombre = w
	sub(/[0-9]*/, "", nombre)

	# Num2Text only numbers! 
	if (length(nombre) == 0)
	{
		strnum = FCT_Convert(w)

		# Format the output (see dtd!)
		gsub("[  ]+", " ", strnum)	# remove multiple spaces
		gsub("^ ", "", strnum)		# remove a starting space
		gsub(" $", "", strnum)		# remove an ending space
		gsub(" ", " </str>\n</wd>\n<wd>\n<str> ", strnum)	# add tags
		strnum = "<str> " strnum " </str>"			# add tags

		# Print
		print strnum > Output
		next
	}
}

(xml == 0) {
	# Verify if the input string is a number
	nombre = $0
	sub(/[\.0-9]*/, "", nombre)

	# Num2Text only numbers! 
	if (length(nombre) == 0)
	{
		strnum = FCT_Convert($0)
		gsub("[  ]+", " ", strnum)	# remove multiple spaces
		gsub("^ ", "", strnum)		# remove a starting space
		print strnum > Output
		next
	}
}

{
	print > Output
}

# ############################## END MAIN ################################# #



