#! /usr/bin/gawk -f
# ######################################################################### #
#                                                                           #
# File:     Num2LetterSP.awk                                                #
# Authors:  Brigitte Bigi                                                   #
# Date:     Avril, 2007                                                     #
# Licence:  GPL                                                             #
# Brief:    Convert numbers of a corpus to the corresponding Spanish string #
# Help:     Num2LetterSP.awk -h                                             # 
#                                                                           #
# ######################################################################### #



# ######################################################################### #
#                                                                           #
# Function: unite                                                           #
# 	Converts numbers from 0 to 9 and returns the corresponding string   #
#                                                                           #
# ######################################################################### #
function unite( number )
{
	_r = ""
	if (number == 9)  _r = "nueve "
	if (number == 8)  _r = "ocho "
	if (number == 7)  _r = "siete "
	if (number == 6)  _r = "seis "
	if (number == 5)  _r = "cinco "
	if (number == 4)  _r = "cuatro "
	if (number == 3)  _r = "tres "
	if (number == 2)  _r = "dos "
	if (number == 1)  _r = "un "
	if (number == 0)
	{
		_str = " cero "
		for(i=1; i<length(number); i++)
			_str = " " _str " cero "
		return _str
	}
	return _r
}


# ######################################################################### #
#                                                                           #
# Function: dizaine                                                         #
# 	Converts numbers from 1 to 99 and returns the corresponding string  #
#                                                                           #
# ######################################################################### #
function dizaine( number )
{
	if (number >= 90 && number <= 99)
	{
		_str = "noventa "
		if (number > 91)
			_str = _str " y " unite(number-90)
	}
	else if (number >= 80 && number <= 89)
	{
		_str = "ochenta "
		if (number > 81)
			_str =  _str " y " unite(number-80)
	}
	else if (number >= 70 && number <= 79)
	{
		_str = "setenta "
		if (number > 71)
			_str = _str " y " unite(number-70)
	}
	else if (number >= 60 && number <= 69)
	{
		_str = "sesenta y "
		if (number == 61)
			_str = _str "uno "
		if (number > 61)
			_str = _str unite(number-60)
	}
	else if (number >= 50 && number <= 59)
	{
		_str = "cincuenta y "
		if (number == 51)
			_str = _str "uno "
		if (number > 51)
			_str = _str unite(number-50)
	}
	else if (number >= 40 && number <= 49)
	{
		_str = "cuarenta y "
		if (number == 41)
			_str = _str "uno "
		if (number > 41)
			_str = _str unite(number-40)
	}
	else if (number >= 30 && number <= 39)
	{
		_str = "treinta y "
		if (number == 31)
			_str = _str "uno "
		if (number > 31)
			_str = _str unite(number-30)
	}
	else if (number >= 20 && number <= 29)
	{
		_str = "veinti"
		if (number == 21)
			_str = _str "uno "
		else
		if (number > 21)
			_str = _str unite(number-20)

	}
	else if (number >= 10 && number <= 19)
	{
		if (number == 10) _r = "diez "
		if (number == 11) _r = "once "
		if (number == 12) _r = "doce "
		if (number == 13) _r = "trece "
		if (number == 14) _r = "catorce "
		if (number == 15) _r = "quince "
		if (number == 16) _r = "diecisis "
		if (number == 17) _r = "diecisiete "
		if (number == 18) _r = "dieciocho "
		if (number == 19) _r = "diecinueve "
		return _r
	}
	else
		_str = unite(number)

	return _str
}	


# ######################################################################### #
#                                                                           #
# Function: centaine                                                        #
# 	Converts numbers from 1 to 999 and returns the corresponding string #
#                                                                           #
# ######################################################################### #
function centaine( number )
{
	if (number >= 100)
	{
		if (number > 900 && number <= 999)
			_str =  "novecientos " dizaine(number-900)
		else if (number == 900)
			_str =  "novecientos"
		else if (number > 800 && number <= 899)
			_str =  "ochocientos " dizaine(number-800)
		else if (number == 800)
			_str =  "ochocientos "
		else if (number > 700 && number <= 799)
			_str =  "sietecientos " dizaine(number-700)
		else if (number == 700)
			_str =  "sietecientos "
		else if (number > 600 && number <= 699)
			_str =  "seiscientos " dizaine(number-600)
		else if (number == 600)
			_str =  "seiscientos "
		else if (number > 500 && number <= 599)
			_str =  "quinientos " dizaine(number-500)
		else if (number == 500)
			_str =  "quinientos "
		else if (number > 400 && number <= 499)
			_str =  "cuatrocientos " dizaine(number-400)
		else if (number == 400)
			_str =  "cuatrocientos "
		else if (number > 300 && number <= 399)
			_str =  "trescientos " dizaine(number-300)
		else if (number == 300)
			_str =  "trescientos "
		else if (number > 200 && number <= 299)
			_str =  "doscientos " dizaine(number-200)
		else if (number == 200)
			_str =  "doscientos "
		else if (number > 100 && number <= 199)
			_str =  "ciento " dizaine(number-100)
		else if (number == 100)
			_str = "cien"
	}
	else
		_str = dizaine(number)
		
	return _str	
}	


# ######################################################################### #
#                                                                           #
# Function: millier                                                         #
# Converts numbers from 1000 to 99999 and returns the corresponding string  #
#                                                                           #
# ######################################################################### #
function millier( number )
{
	if (number < 1000) _str = centaine(number)

	# Milliers
	# --------
	if ( number == 1000 )
		_str = "mil "
	
	if ( ( number > 1000 )  &&  ( number < 2000 ) )
		_str = "mil " centaine(number%1000)

	if ( ( number >= 2000 ) && ( number < 10000 ) )
	{
		if ( (number%1000) != 0 )
			_str = unite(int (number/1000)) " mil " centaine(number%1000)
		else
			_str = unite(int (number/1000)) " mil "
	}


	# Dizaines de milliers
	# --------------------
	if (number == 10000)
		_str = "diez miles "
	
	if (number > 10000 && number < 100000)
	{
		if ( (number%1000) != 0 )
			_str = dizaine(int (number/1000)) " mil " centaine(number%1000)
		else
			_str = dizaine(int (number/1000)) " mil"
	}
	

	# Centaines de milliers
	# ---------------------
	if (number == 100000)
		_str = "ciento mil"
	
	if (number >= 100000 && number < 1000000)
	{
		if ( (number%1000) != 0 )
			_str = centaine(int (number/1000)) "mil " centaine(int (number%1000))
		else
			_str = centaine(int (number/1000)) "mil "
	}

	return _str
}


# ######################################################################### #
#                                                                           #
# Function: FCT_Num2text                                                    #
#	Converts all numbers and returns the corresponding string           #
#                                                                           #
#  number is the number to convert                                          #
#                                                                           #
# ######################################################################### #
function FCT_Num2text( number )
{
	if (number < 1000000)
		_str = millier(number)

	# Millions
	# --------
	if (number >= 1000000 && number < 2000000)
		_str = "un millon " millier(int (number%1000000))

	if (number >= 2000000 && number < 10000000)
		_str = unite(int (number/1000000)) "millones " millier(int (number%1000000))


	# Dizaines de millions
	# --------------------
	if (number == 10000000)
		_str = "diez milliones"

	if (number > 10000000 && number < 100000000)
		_str = dizaine(int (number/1000000)) "millones " millier(int (number%1000000))
		

	# Centaines de millions
	# ---------------------
	if (number == 100000000)
		_str = "ciento milliones"

	if (number > 100000000 && number < 1000000000)
		_str = centaine(int (number/1000000)) "millones " millier(int (number%1000000))

	if (number > 1000000000)
		return number

	return _str
}


# ######################################################################### #
#                                                                           #
# Function Name: FCT_Usage                                                  #
#                                                                           #
# Description:                                                              #
#     Print the usage and exit the program                                  #
#                                                                           #
# ######################################################################### #
function FCT_Usage ()
{
        printf "Usage: Num2LetterSP.awk [options]\n\n"
        printf "Where options include:\n"
	printf "  -o filename  write the output to this file (default: stdout)\n"
	printf "  -noxml       input is one word a line (instead of XML)\n"
        printf "  -h	print  this help\n"
	printf "\n"
	printf " input number: stdin\n"
	printf " output text:  stdout\n"
        exit
}


# ######################################################################### #
#                                                                           #
# Function Name: FCT_CommandLine                                            #
#                                                                           #
# Description:                                                              #
#     Get arguments from the command line                                   #
#                                                                           #
# ######################################################################### #
function FCT_CommandLine(argc, argv)
{
        entry = 0
        for (i=1; i<argc; i++)
        {
                if (argv[i] == "-h")
                        FCT_Usage()
                else
                if (argv[i] == "-noxml")
                {
                        xml = 0
                        i++
                }
                else
                if (argv[i] == "-o")
                {
                        Output = argv[i+1]
                        i++
                }
                else
		if (argv[i] ~ /^\-/)
		{
			printf "invalid option -- %s\n", argv[i] > "/dev/stderr"
			FCT_Usage()
		}
        }
}

# ######################################################################### #
#                                                                           #
# Function Name: FCT_Convert                                                #
#                                                                           #
# Description:                                                              #
#     Get a number and return a string                                      #
#                                                                           #
# ######################################################################### #

function FCT_Convert ( w )
{
	_w = w
	_strnum = ""

	# Numbers starting by one or more "0"
	if (match(_w,/^0/))
	{
		while (match(_w,/^0/))
		{
			_strnum = _strnum " cero "
			sub(/^0/, "", _w)
		}
	}
	gsub(/\./, "",_w)
	if (length(_w) > 0)
		_strnum = _strnum FCT_Num2text(int(_w))
	return _strnum
}


# ############################# BEGIN MAIN ################################ #

BEGIN {
	Output      = "/dev/stdout"	# output file name
	xml         = 1			# xml input ??? (yes is default)
        FCT_CommandLine(ARGC, ARGV)	# extract parameters
        delete ARGV
        ARGC=1
}

/<str>/ && (xml == 1) {
        # Get the element
        s = index($0, ">")				# start: 1st >
        e = index(substr($0, s+1, length($0)), "<")	# end: 1st < after start
        w = substr($0, s+1, e-1)			# word: $0 from start to end
        gsub(/ /, "", w)				# remove spaces

	# Verify if the input string is a number
	nombre = w
	sub(/[\.0-9]*/, "", nombre)

	# Num2Text only numbers! 
	if (length(nombre) == 0)
	{
		strnum = FCT_Convert(w)

		# Format the output (see dtd!)
		gsub("[  ]+", " ", strnum)	# remove multiple spaces
		gsub("^ ", "", strnum)		# remove a starting space
		gsub(" $", "", strnum)		# remove an ending space
		gsub(" ", " </str>\n</wd>\n<wd>\n<str> ", strnum)	# add tags
		strnum = "<str> " strnum " </str>"			# add tags

		# Print
		print strnum > Output
		next
	}
}

(xml == 0) {
	# Verify if the input string is a number
	nombre = $0
	sub(/[\.0-9]*/, "", nombre)

	# Num2Text only numbers! 
	if (length(nombre) == 0)
	{
		strnum = FCT_Convert($0)
		gsub("[  ]+", " ", strnum)	# remove multiple spaces
		gsub("^ ", "", strnum)		# remove a starting space
		print strnum > Output
		next
	}
}

{
	print > Output
}

# ############################## END MAIN ################################# #


