#! /usr/bin/gawk -f
# ######################################################################### #
#                                                                           #
# File:     Num2LetterVN.awk                                                #
# Authors:  Brigitte BIGI, Viet Bac LE                                      #
# Date:     Oct, 2007                                                       #
# Licence:  GPL                                                             #
# Brief:    Convert numbers of a corpus to the corresp. Vietnamese string   #
# Help:     Num2LetterVN.awk -h                                             #
#                                                                           #
# ######################################################################### #

# ######################################################################### #
#                                                                           #
# Function: dizaine                                                         #
# 	Converts numbers from 10 to 99 and returns the corresponding string #
#                                                                           #
# ######################################################################### #
function dizaine( number )
{
	if ((number >= 20) && (number <= 99))
	{
		_str1 = arr_unite[int(number/10)+1] " " arr_unite_dizaine[2]
		if ((number%10) != 0) 
			_str1 = _str1 " " arr_unite2[int(number%10)+1]
	}
	else if ((number >= 10) && (number <= 19))
	{
		_str1 = arr_unite_except[2]
		if ((number%10) != 0) 
			_str1 = _str1 " " arr_unite3[int(number%10)+1]		
	}
	else 
		_str1 = arr_unite_except[3] " " arr_unite1[number+1]
	
	return _str1;	
	
}

# ######################################################################### #
#                                                                           #
# Function: centaine                                                        #
#   Converts numbers from 100 to 999 and returns the corresponding string   #
#                                                                           #
# ######################################################################### #
function centaine( number )
{
	if (number > 0)
	{
		_str2 = arr_unite[int(number/100)+1] " " arr_unite_dizaine[3]
		if ((number%100) != 0) _str2 = _str2 " " dizaine(int(number%100))		
	}
			
	return _str2;
}

# ######################################################################### #
#                                                                           #
# Function: milliers                                                        #
# Converts numbers from 1000 and returns the corresponding string           #
#                                                                           #
# ######################################################################### #
function milliers( number )
{
	k = 1
	debut = 1
	size = length(number)
	_str3 = ""
	
	for (i=1; i<=int((size-1)/3); i++) 
		k *= 1000

	do {
		millier = int(number/k)
		if (debut) {
			tmp = (size-1)%3+1
			if (tmp == 1) _str3 = _str3 arr_unite[millier+1] " " arr_unite_millier[int((size-1)/3)+1]
			else if (tmp == 2) _str3 = _str3 dizaine(millier) " " arr_unite_millier[int((size-1)/3)+1]
			else _str3 = _str3 centaine(millier) " " arr_unite_millier[int((size-1)/3)+1]
			debut = 0
		}
		else _str3 = _str3 centaine(millier) " " arr_unite_millier[int((size-1)/3)+1]

		do {
			number = int(number%k)
      			k = int(k/1000)
			size -= 3			
		} while ((number<k) && (k>=1));
		
		if (number > 0) 
			_str3 = _str3 " "	
	} while (size > 3)
	
	if (number > 0) _str3 = _str3 centaine(number)
	
	return _str3
}


# ######################################################################### #
#                                                                           #
# Function: FCT_Num2text                                                    #
#	Converts all numbers and returns the corresponding string           #
#                                                                           #
#  number is the number to convert                                          #
#                                                                           #
# ######################################################################### #
function FCT_Num2text( number )
{
	str_unite = "không/một/hai/ba/bốn/năm/sáu/bảy/tám/chín"
	split(str_unite, arr_unite, "/")
	
  	str_unite1 = "/một/hai/ba/tư/năm/sáu/bảy/tám/chín" 		#ex: 102
  	split(str_unite1, arr_unite1, "/")
  
  	str_unite2 = "/mốt/hai/ba/tư/lăm/sáu/bảy/tám/chín"		#ex: 24
  	split(str_unite2, arr_unite2, "/")
  
	str_unite3 = "/một/hai/ba/bốn/lăm/sáu/bảy/tám/chín"	#ex: 15
	split(str_unite3, arr_unite3, "/")
	
	str_unite_dizaine = "/mươi/trăm"
	split(str_unite_dizaine, arr_unite_dizaine, "/")
	
	str_unite_millier = "/nghìn/triệu/tỷ/nghìn tỷ/triệu tỷ/tỷ tỷ"
	split(str_unite_millier, arr_unite_millier, "/")
	
	str_unite_except = "không/mười/linh"
	split(str_unite_except, arr_unite_except, "/")
		
	l = length(number)

	if ( l == 1 ) _str4 = arr_unite[number+1]
	else if ( l == 2 ) _str4 = dizaine(number)
	else if ( l == 3 ) _str4 = centaine(number)
	else if ( l <= 21) _str4 = milliers(number)
	else _str4 = ""	# Out of number :D

	return _str4	
}


# ######################################################################### #
#                                                                           #
# Function Name: FCT_Usage                                                  #
#                                                                           #
# Description:                                                              #
#     Print the usage and exit the program                                  #
#                                                                           #
# ######################################################################### #
function FCT_Usage ()
{
  	printf "Usage: Num2LetterVN.awk [options]\n\n"
  	printf "Where options include:\n"
	printf "  -o filename  write the output to this file (default: stdout)\n"
	printf "  -noxml       input is one word a line (instead of XML)\n"
  	printf "  -h	print  this help\n"
	printf "\n"
	printf " input number: stdin\n"
	printf " output text:  stdout\n"
        exit
}


# ######################################################################### #
#                                                                           #
# Function Name: FCT_CommandLine                                            #
#                                                                           #
# Description:                                                              #
#     Get arguments from the command line                                   #
#                                                                           #
# ######################################################################### #
function FCT_CommandLine(argc, argv)
{
        entry = 0
        for (i=1; i<argc; i++)
        {
                if (argv[i] == "-h")
                        FCT_Usage()
                else
                if (argv[i] == "-noxml")
                {
                        xml = 0
                        i++
                }
                else
                if (argv[i] == "-o")
                {
                        Output = argv[i+1]
                        i++
                }
                else
		if (argv[i] ~ /^\-/)
		{
			printf "invalid option -- %s\n", argv[i] > "/dev/stderr"
			FCT_Usage()
		}
        }
}

# ######################################################################### #
#                                                                           #
# Function Name: FCT_Convert                                                #
#                                                                           #
# Description:                                                              #
#     Get a number and return a string                                      #
#                                                                           #
# ######################################################################### #

function FCT_Convert ( w )
{
	_w = w
	_strnum = ""

	# Numbers starting by one or more "0"
	if (match(_w,/^0/))
	{
		while (match(_w,/^0/))
		{
			_strnum = _strnum " không "
			sub(/^0/, "", _w)
		}
	}
	gsub(/\./, "",_w)
	if (length(_w) > 0)
		_strnum = _strnum FCT_Num2text(int(_w))
	return _strnum
}



# ############################# BEGIN MAIN ################################ #

BEGIN {
	Output      = "/dev/stdout"	# output file name
	xml         = 1			# xml input ??? (yes is default)
        FCT_CommandLine(ARGC, ARGV)	# extract parameters
        delete ARGV
        ARGC=1
}

/<str>/ {
        # Get the element
        s = index($0, ">")				# start: 1st >
        e = index(substr($0, s+1, length($0)), "<")	# end: 1st < after start
        w = substr($0, s+1, e-1)			# word: $0 from start to end
        gsub(/ /, "", w)				# remove spaces

	# Verify if the input string is a number
	nombre = w
	sub(/[0-9]*/, "", nombre)

	# Num2Text only numbers! 
	if (length(nombre) == 0)
	{
		strnum = FCT_Convert(w)

		# Format the output (see dtd!)
		gsub("[  ]+", " ", strnum)	# remove multiple spaces
		gsub("^ ", "", strnum)		# remove a starting space
		gsub(" $", "", strnum)		# remove an ending space
		gsub(" ", " </str>\n</wd>\n<wd>\n<str> ", strnum)	# add tags
		strnum = "<str> " strnum " </str>"			# add tags
		
		# Print
		print strnum > Output
		
		next
	}
}

(xml == 0) {
	# Verify if the input string is a number
	nombre = $0
	sub(/[\.0-9]*/, "", nombre)

	# Num2Text only numbers! 
	if (length(nombre) == 0)
	{
		strnum = FCT_Convert($0)
		gsub("[  ]+", " ", strnum)	# remove multiple spaces
		gsub("^ ", "", strnum)		# remove a starting space
		print strnum > Output
		next
	}
}

{
	print > Output
}

# ############################## END MAIN ################################# #


