#! /usr/bin/gawk -f
# ######################################################################### #
#                                                                           #
# File:     Remove.awk                                                      #
# Authors:  Brigitte Bigi                                                   #
# Date:     April, 2007                                                     #
# Licence:  GPL                                                             #
# Brief:    Load a vocabulary and remove these words/lems/pos               #
# Help:     Remove.awk -h                                                   #
#                                                                           #
# ######################################################################### #
 


# ######################################################################### #
#                                                                           #
#  Function FCT_LoadVocab                                                   #
#                                                                           #
#  Description: Load a vocabulary from a file                               #
#                                                                           #
#  Parameters:                                                              #
#       file is the file name of the vocabulary                             #
#                                                                           #
#  Output:                                                                  #
#       TAB_Voc is a string array                                           #
#                                                                           #
#  Return value:                                                            #
#       integer value: the number of words (TAB_Voc size)                   #
#                                                                           #
# ######################################################################### #
function FCT_LoadVocab ( file )
{
	_ind = 0
	while ( getline < file  )
	{
		TAB_Voc[$0]=_ind
		_ind++
	}
	return _ind
}
# ######################################################################### #



# ######################################################################### #
#                                                                           #
#  Function FCT_IsOOV                                                       #
#                                                                           #
#  Return value:                                                            #
#       boolean value: 1 if word is in TAB_Voc, 0 else                      #
#                                                                           #
# ######################################################################### #
function FCT_IsOOV ( word )
{
	if (TAB_Voc[word] != "") return 0
	return 1
}


# ######################################################################### #
#                                                                           #
# Function Name: FCT_Usage                                                  #
#                                                                           #
# Description: Print the usage and exit the program                         #
#                                                                           #
# ######################################################################### #
function FCT_Usage ()
{
        printf "Usage: Remove.awk [options]\n\n"
        printf "Where options include:\n"
	printf "  -o filename   write the output to this file (default: stdout)\n"
        printf "  -V filename   read vocabulary from this file\n"
	printf "  -e            remove an element\n"
	printf "  -w            remove words (default)\n"
	printf "  -l            remove lemmas\n"
	printf "  -p            remove pos tagg\n"
        printf "  -h            print this help\n"
        printf "\n"
        printf " input text:  stdin\n"
        printf " output text: stdout (except if -o)\n"
        exit
}


# ######################################################################### #
#                                                                           #
# Function Name: FCT_CommandLine                                            #
#                                                                           #
# Description: Gets arguments from the command line                         #
#                                                                           #
# ######################################################################### #
function FCT_CommandLine(argc, argv)
{
        for (i=1; i<argc; i++)
        {
                if (argv[i] == "-h")
                        FCT_Usage()
                else
                if (argv[i] == "-V")
                {
                        vocabsize = FCT_LoadVocab(argv[i+1])
                        i++
                }
                else
		if (argv[i] == "-w")
		{
			remove = 1
		}
                else
		if (argv[i] == "-l")
		{
			remove = 2
		}
                else
		if (argv[i] == "-p")
		{
			remove = 3
		}
		else
                if (argv[i] == "-o")
                {
                        Output = argv[i+1]
                        i++
                }
                else
                if (argv[i] ~ /^\-/)
                {
                        printf "invalid option -- %s\n", argv[i] > "/dev/stderr"
                        FCT_Usage()
                }
        }
}

# ######################################################################### #
#                                                                           #
# Function Name: FCT_Remove                                                 #
#                                                                           #
# Description: Remove or not the line depending on the string               #
#                                                                           #
# ######################################################################### #
function FCT_Remove (str)
{
        # Get the string
        _s = index(str, ">")				# start: 1st >
        _e = index(substr(str, _s+1, length(str)), "<")	# end: 1st < after start
        _w = substr(str, _s+1, _e-1)			# word: $0 from start to end
        gsub(/ /, "", _w)				# remove spaces

        # Remove ?
        return FCT_IsOOV(_w)
}

# ############################# BEGIN MAIN ################################ #

BEGIN{
        if ( ARGC == 1 )		# Need, at least the -V option
                FCT_Usage()
	wd          = 1
	oov         = 0			# boolean (remove ?)
        vocabsize   = 0			# vocabulary size
	remove      = 1			# remove words (2:lemmas;3:pos taggs)
	Output      = "/dev/stdout"	# output file name
        FCT_CommandLine(ARGC, ARGV)	# extract parameters
        delete ARGV
        ARGC=1
}

# New word
/<wd[ >]/ {
	wd  = 1
	oov = 0
	# This element contains tags (word, lem, or pos)
	TabWd[wd] = $0	
	wd++
	next
}

# End of word: do anything or print!
/<\/wd>/ {
	if (oov == 1)
	{
		for (i=1; i<wd; i++)
			print TabWd[i] > Output
		print > Output
	}
	delete TabWd
	wd = 1
	next
}

/<str>/ && (remove == 1) {
	oov = FCT_Remove($0)
}

/<lem>/ && (remove == 2) {
	oov = FCT_Remove($0)
}

/<pos>/ && (remove == 3) {
	oov = FCT_Remove($0)
}

# all other cases than <wd> and </wd>
{
	if (wd > 1)
	{
		TabWd[wd] = $0
		wd++
	}
	else
		# other lines must NOT be changed
		print > Output
}

# ############################## END MAIN ################################# #

