#! /usr/bin/gawk -f
# ##################################################################### #
#									#
# Program Name:  SplitOrig2Wd.awk					#
# Author:	 Brigitte Bigi						#
# Licence:       GPL							#
# Date:	         April, 2006						#
# Brief:	 Cut the <orig> to add <wd> tags			#
# Help:	         SplitOrig2Wd.awk -h					#
# ##################################################################### #


# ##################################################################### #
#									#
# Function Name: FCT_Usage						#
#									#
# Description:								#
#     Print the usage and exit the program				#
#									#
# ##################################################################### #
function FCT_Usage ()
{
	printf "Usage: SplitOrig2Wd.awk [options]\n\n"
	printf "Where options include:\n"
	printf "  -e id	       give the id to elts (default: elt)\n"
	printf "  -l lang      give a specific language (CN or nothing)\n"
	printf "  -n           print <s> null </s> if the sentence is empty\n"
	printf "  -o filename  write the output to this file (default: stdout)\n"
	printf "  -h	       print this help\n"
	printf "\n"
	printf " input text:  stdin\n"
	printf " output text: stdout\n"
	exit
}


# ##################################################################### #
#									#
# Function Name: FCT_CommandLine					#
#									#
# Description:							  	#
#     Get arguments from the command line				#
#									#
# ##################################################################### #
function FCT_CommandLine(argc, argv)
{
	entry = 0
	for (i=1; i<argc; i++)
	{
		if (argv[i] == "-h")
			FCT_Usage()
		else
		if (argv[i] == "-e")
		{
			EltsID = argv[i+1]
			i++
		}
		else
		if (argv[i] == "-o")
		{
			Output = argv[i+1]
			i++
		}
                else
                if (argv[i] == "-n")
                {
                        nullsent = 1
                }
		else
		if (argv[i] == "-l")
		{
			lang = argv[i+1]
			i++
		}
		else
		if (argv[i] ~ /^\-/)
		{
			printf "invalid option -- %s\n", argv[i] > "/dev/stderr"
			FCT_Usage()
		}
	}
}


# ############################# BEGIN MAIN ############################# #


BEGIN {
	lang        = none		# Default language
	EltsID      = "elt"		# elts ID
	nullsent    = 0			# boolean: print a null sentence if no <wd> were fouunded
	Output      = "/dev/stdout"     # output file name
	FCT_CommandLine(ARGC, ARGV)     # extract parameters
	delete ARGV
	ARGC=1

	orig = ""
	inorig = 0
}

/<[\/]?sent>/ {
	next
}

/<orig>/ {
	# boolean: inside the original string ?
	# (a sentence can be written over many lines)
	inorig = 1
	next
}

/<\/orig>/ {
	# not an empty sentence... print orig and elts
	if (length(orig) > 0)
	{
		print "<sent>" > Output
		print "<orig>" > Output
		print orig     > Output
		print "</orig>" > Output
		printf "<elts id=\"%s\">\n",EltsID > Output

		if (lang == "CN")
		{
			str=gensub(/(.)/, "\\\\\\1", "g", orig)
			cmd = ("echo " str " | $CTTK/bin/SplitCharsCN.py")
			cmd | getline orig
			close(cmd)
		}

		# -------------------------------------------------
		# Replace ^M by space
		gsub(/\r/, " ", orig)

		# Remove multiple spaces
		gsub(/[ ]+/, " ", orig)

		# Replace spaces by underscore
		gsub(/ /, "_", orig)

		# Remove multiple spaces
		gsub(/[_]+/, "_", orig)
		# --------------------------------------------------
		
		# Replace -- by only one _
		while ( match(orig,/--/) )
			gsub(/--/, "_", orig)

		# Add a space if some punctuations are sticked to a word
		orig = gensub(/([$+>^@}\(\){~|=])([[:alnum:]])/, "_\\1_\\2", "g", orig)
		orig = gensub(/([[:alnum:]])([$+>^@}\(\){~|=])/, "\\1_\\2_", "g", orig)

		# Split numbers if sticked to characters
		orig = gensub(/([[:digit:]])([^.,[:digit:]])/, "\\1_\\2", "g", orig)
		orig = gensub(/([^[:digit:].,])([[:digit:]])/, "\\1_\\2", "g", orig)

		# split some punct.
		gsub(/\]\[/, "]_[", orig)

		# Split dots if sticked to a word 
	    	orig = gensub(/ \.([[:alpha:]-])/, "._\\1", "g", orig)
		orig = gensub(/^\.([[:alpha:]-])/, "._\\1", "g", orig)

		# --------------------------------------------------

		# --------------------------------------------------
		# and voil!
		gsub(/__/, "_",orig)
		gsub(/^_/, "", orig)
		gsub(/_$/, "", orig)

		nb = split(orig, TAB_Str, "_")
		for (i=1; i<=nb; i++)
			printf "<wd>\n<str> %s </str>\n</wd>\n", TAB_Str[i] > Output
		print "</elts>" > Output
		print "</sent>" > Output
		# --------------------------------------------------

		orig = ""
		delete TAB_Str
	}
	else
	if (nullsent == 1)
	{
		print "<sent>"  > Output
		print "<orig>"  > Output
		print "null"    > Output
		print "</orig>" > Output
		print "</sent>"  > Output
	}
	inorig=0
	next
}


(inorig==1) {
	# inside orig
	# do some cleaning... 
	gsub("[  ]+", " ")
	gsub("^ ", "")
	gsub("^	", "")

	# and remember it!
	if (length($0) > 0)
		orig = orig $0

	# dont print... get next line
	next
}

{
	print > Output
}
