#! /bin/awk -f
# ######################################################################### #
#                                                                           #
# File:     Text2Counts.awk                                                 #
# Authors:  Brigitte Bigi                                                   #
# Date:     Nov, 2003                                                       #
# Licence:  GPL                                                             #
# Brief:    Print list of word counts from a corpus                         #
# Help:     Text2Counts.awk -h                                              #
#                                                                           #
# ######################################################################### #

# ######################################################################### #
#                                                                           #
# Function Name: FCT_Usage                                                  #
#                                                                           #
# Description:                                                              #
#     Print the usage and exit the program                                  #
#                                                                           #
# ######################################################################### #
function FCT_Usage ()
{
        printf "Usage: Text2Counts.awk [options]\n\n"
        printf "Where options include:\n"
        printf "  -h	print this help\n"
	printf "\n"
	printf " input text:  stdin\n"
	printf " output text: stdout\n"
        exit
}


# ######################################################################### #
#                                                                           #
# Function Name: FCT_CommandLine                                            #
#                                                                           #
# Description:                                                              #
#     Get arguments from the command line                                   #
#                                                                           #
# ######################################################################### #
function FCT_CommandLine(argc, argv)
{
        entry = 0
        for (i=1; i<argc; i++)
        {
                if (argv[i] == "-h")
                        FCT_Usage()
                else
		if (argv[i] ~ /^\-/)
		{
			printf "invalid option -- %s\n", ARGV[i] > "/dev/stderr"
			FCT_Usage()
		}
		else
			break
        }
}


# ############################# BEGIN MAIN ################################ #

BEGIN {
        FCT_CommandLine(ARGC, ARGV)
        delete ARGV
        ARGC=1
}

{
	# Create a tab word/count
	for (i = 1; i <= NF; i++)
		TAB_Counts[$i]++
}

END {
	# sort tab and print on stdout
	sort = "sort -k 2 -nr"
	for (word in TAB_Counts)
		printf "%s\t%d\n", word, TAB_Counts[word] | sort
	close(sort)
}

################################ END MAIN ###################################

